﻿using UnityEngine;
using System.Collections;

public class MouseManager : MonoBehaviour {

	// Use this for initialization
	void Start () {
	
	}

    public GameObject thePrefabToSpawn;

    public LayerMask snapPointLayerMask;
	
	// Update is called once per frame
	void Update () {
	
        // Was the mouse pressed down this frame?

        if(Input.GetMouseButtonDown(0))
        {
            // Yes, the left mouse button was pressed this frame
            // Is the mouse over a cube

            Camera theCamera = Camera.main;

            Ray ray = theCamera.ScreenPointToRay( Input.mousePosition );

            RaycastHit hitInfo;

            if( Physics.Raycast( ray, out hitInfo ) )
            {
                // Did we click on a snap point?
                // Our snap points are always going to be on the SnapPoint physics layer

                // The normal integer number value of 8 is represented thusly in binary:
                //
                //      0000000000000000001000
                //
                // When you're talking about things like "LayerMasks" "BitMasks" "Flags"
                //
                //      00000000000000100010000 (in this case "Water" and "SnapPoint" are on)
                //      Would represent something like the 4th item in the list is enabled
                //
                // Bitwise operators and comparison
                //
                //  &: This kind of compares two integers and returns a new integer where
                //     both had their bits enabled
                //      Example:
                //              10110 
                //            & 10011
                //              ----
                //              10010
                //
                //  |: This is a bitwise or
                //
                //              10110 
                //            | 10011
                //              -----
                //              10111
                //
                //
                //  <<: The left-shift operator.
                //
                //     1 << 8
                //
                //    0000000000000000001
                //    0000000000100000000
                //

                int maskForThisHitObject = 1 << hitInfo.collider.gameObject.layer;


                if( (maskForThisHitObject & snapPointLayerMask) != 0  )
                {
                    // Now let's spawn a new object

                    Vector3 spawnSpot = hitInfo.collider.transform.position + hitInfo.normal;

                    Quaternion spawnRotation = Quaternion.FromToRotation( Vector3.forward, hitInfo.normal );

                    Instantiate( thePrefabToSpawn, spawnSpot, spawnRotation );

                }


            }
        }


	}


}
